@@ -7,11 +7,12 @@ __license__ = 'MIT'
 
 import warnings
 from ast import literal_eval
-from typing import Any, Callable, List, Optional, Type, Union
+from typing import Any, Callable, List, Optional, Tuple, Type, Union
 
-import readchar
 from rich.console import Console
 from rich.live import Live
+from yakh import get_key
+from yakh.key import Keys
 
 from beaupy._internals import (
     ConversionError,
@@ -25,31 +26,37 @@ from beaupy._internals import (
 
 console = Console()
 
-readchar.config.INTERRUPT_KEYS = []
-
 
 class DefaultKeys:
     """A map of default keybindings.
 
     Attributes:
-        interrupt(List[str]): Keys that cause a keyboard interrupt.
-        select(List[str]): Keys that trigger list element selection.
-        confirm(List[str]): Keys that trigger list confirmation.
-        delete(List[str]): Keys that trigger character deletion.
-        down(List[str]): Keys that select the element below.
-        up(List[str]): Keys that select the element above.
+        escape(List[Union[Tuple[int, ...], str]]): Keys that escape the current context.
+        select(List[Union[Tuple[int, ...], str]]): Keys that trigger list element selection.
+        confirm(List[Union[Tuple[int, ...], str]]): Keys that trigger list confirmation.
+        backspace(List[Union[Tuple[int, ...], str]]): Keys that trigger deletion of the previous character.
+        delete(List[Union[Tuple[int, ...], str]]): Keys that trigger deletion of the next character.
+        down(List[Union[Tuple[int, ...], str]]): Keys that select the element below.
+        up(List[Union[Tuple[int, ...], str]]): Keys that select the element above.
+        left(List[Union[Tuple[int, ...], str]]): Keys that select the element to the left.
+        right(List[Union[Tuple[int, ...], str]]): Keys that select the element to the right.
+        home(List[Union[Tuple[int, ...], str]]): Keys that move to the beginning of the context.
+        end(List[Union[Tuple[int, ...], str]]): Keys that move to the end of the context.
     """
 
-    interrupt: List[str] = [readchar.key.CTRL_C, readchar.key.CTRL_D]
-    select: List[str] = [readchar.key.SPACE]
-    confirm: List[str] = [readchar.key.ENTER]
-    delete: List[str] = [readchar.key.BACKSPACE]
-    down: List[str] = [readchar.key.DOWN]
-    up: List[str] = [readchar.key.UP]
-    left: List[str] = [readchar.key.LEFT]
-    right: List[str] = [readchar.key.RIGHT]
-    home: List[str] = [readchar.key.HOME]
-    end: List[str] = [readchar.key.END]
+    escape: List[Union[Tuple[int, ...], str]] = [Keys.ESC]
+    select: List[Union[Tuple[int, ...], str]] = [' ']
+    confirm: List[Union[Tuple[int, ...], str]] = [Keys.ENTER]
+    backspace: List[Union[Tuple[int, ...], str]] = [Keys.BACKSPACE]
+    delete: List[Union[Tuple[int, ...], str]] = [Keys.DELETE]
+    down: List[Union[Tuple[int, ...], str]] = [Keys.DOWN_ARROW, Keys.NUMPAD_DOWN_ARROW]
+    up: List[Union[Tuple[int, ...], str]] = [Keys.UP_ARROW, Keys.NUMPAD_UP_ARROW]
+    left: List[Union[Tuple[int, ...], str]] = [Keys.LEFT_ARROW, Keys.NUMPAD_LEFT_ARROW]
+    right: List[Union[Tuple[int, ...], str]] = [Keys.RIGHT_ARROW, Keys.NUMPAD_RIGHT_ARROW]
+    tab: List[Union[Tuple[int, ...], str]] = [Keys.TAB]
+    home: List[Union[Tuple[int, ...], str]] = [Keys.HOME]
+    end: List[Union[Tuple[int, ...], str]] = [Keys.END]
+    interrupt: List[Union[Tuple[int, ...], str]] = [Keys.CTRL_C]
 
 
 class Config:
@@ -73,6 +80,7 @@ def prompt(
     secure: bool = False,
     raise_validation_fail: bool = True,
     raise_type_conversion_fail: bool = True,
+    initial_value: Optional[str] = None,
 ) -> TargetType:
     """Function that prompts the user for written input
 
@@ -85,6 +93,7 @@ def prompt(
                                                 the error will be reported onto the console. Defaults to True.
         raise_type_conversion_fail (bool, optional): If True, invalid inputs will raise `rich.internals.ConversionError`, else
                                                      the error will be reported onto the console. Defaults to True.
+        initial_value (str, optional): If present, the value is placed in the prompt as the default value.
 
     Raises:
         ValidationError: Raised if validation with provided validator fails
@@ -96,15 +105,19 @@ def prompt(
     """
     rendered = ''
     with _cursor_hidden(console), Live(rendered, console=console, auto_refresh=False, transient=True) as live:
-        value: List[str] = []
-        cursor_index = 0
+        value: List[str] = [*initial_value] if initial_value else []
+        cursor_index = len(initial_value) if initial_value else 0
         error: str = ''
         while True:
             rendered = _render_prompt(secure, value, prompt, cursor_index, error)
             error = ''
             _update_rendered(live, rendered)
-            keypress = readchar.readkey()
-            if keypress in DefaultKeys.confirm:
+            keypress = get_key()
+            if keypress in DefaultKeys.interrupt:
+                if Config.raise_on_interrupt:
+                    raise KeyboardInterrupt()
+                return None
+            elif keypress in DefaultKeys.confirm:
                 str_value = ''.join(value)
                 try:
                     if target_type is bool:
@@ -123,7 +136,7 @@ def prompt(
                     error = f"Input {'<secure_input>' if secure else '`'+str_value+'`'} cannot be converted to type `{target_type}`"
                     if raise_type_conversion_fail:
                         raise ConversionError(error) from None
-            elif keypress in DefaultKeys.delete:
+            elif keypress in DefaultKeys.backspace:
                 if cursor_index > 0:
                     cursor_index -= 1
                     del value[cursor_index]
@@ -133,12 +146,19 @@ def prompt(
             elif keypress in DefaultKeys.right:
                 if cursor_index < len(value):
                     cursor_index += 1
-            elif keypress in DefaultKeys.interrupt:
-                if Config.raise_on_interrupt:
-                    raise KeyboardInterrupt
+            elif keypress in DefaultKeys.escape:
                 return None
+            elif keypress in DefaultKeys.up + DefaultKeys.down:
+                pass
+            elif keypress in DefaultKeys.home:
+                cursor_index = 0
+            elif keypress in DefaultKeys.end:
+                cursor_index = len(value)
+            elif keypress in DefaultKeys.delete:
+                if cursor_index < len(value):
+                    del value[cursor_index]
             else:
-                value.insert(cursor_index, keypress)
+                value.insert(cursor_index, str(keypress))
                 cursor_index += 1
 
 
@@ -146,7 +166,7 @@ Selection = Union[int, Any]
 
 
 def select(
-    options: List[Any],
+    options: List[Union[Tuple[int, ...], str]],
     preprocessor: Callable[[Any], Any] = lambda val: val,
     cursor: str = '>',
     cursor_style: str = 'pink1',
@@ -157,7 +177,7 @@ def select(
     """A prompt that allows selecting one option from a list of options
 
     Args:
-        options (List[Any]): A list of options to select from
+        options (List[Union[Tuple[int, ...], str]]): A list of options to select from
         preprocessor (Callable[[Any], Any]): A callable that can be used to preprocess the list of options prior to printing.
                                              For example, if you passed a `Person` object with `name` attribute, preprocessor
                                              could be `lambda person: person.name` to just show the content of `name` attribute
@@ -201,9 +221,12 @@ def select(
                 + '\n\n(Confirm with [bold]enter[/bold])'  # noqa: W503
             )
             _update_rendered(live, rendered)
-            keypress = readchar.readkey()
-
-            if keypress in DefaultKeys.up:
+            keypress = get_key()
+            if keypress in DefaultKeys.interrupt:
+                if Config.raise_on_interrupt:
+                    raise KeyboardInterrupt()
+                return None
+            elif keypress in DefaultKeys.up:
                 if index > 0:
                     index -= 1
             elif keypress in DefaultKeys.down:
@@ -213,9 +236,7 @@ def select(
                 if return_index:
                     return index
                 return options[index]
-            elif keypress in DefaultKeys.interrupt:
-                if Config.raise_on_interrupt:
-                    raise KeyboardInterrupt
+            elif keypress in DefaultKeys.escape:
                 return None
 
 
@@ -223,7 +244,7 @@ Selections = List[Selection]
 
 
 def select_multiple(
-    options: List[Any],
+    options: List[Union[Tuple[int, ...], str]],
     preprocessor: Callable[[Any], Any] = lambda val: val,
     tick_character: str = '✓',
     tick_style: str = 'pink1',
@@ -238,7 +259,7 @@ def select_multiple(
     """A prompt that allows selecting multiple options from a list of options
 
     Args:
-        options (List[Any]): A list of options to select from
+        options (List[Union[Tuple[int, ...], str]]): A list of options to select from
         preprocessor (Callable[[Any], Any]): A callable that can be used to preprocess the list of options prior to printing.
                                              For example, if you passed a `Person` object with `name` attribute, preprocessor
                                              could be `lambda person: person.name` to just show the content of `name` attribute
@@ -301,9 +322,12 @@ def select_multiple(
                 rendered = f'{rendered}\n[red]Error:[/red] {error_message}'
                 error_message = ''
             _update_rendered(live, rendered)
-            keypress = readchar.readkey()
-
-            if keypress in DefaultKeys.up:
+            keypress = get_key()
+            if keypress in DefaultKeys.interrupt:
+                if Config.raise_on_interrupt:
+                    raise KeyboardInterrupt()
+                return []
+            elif keypress in DefaultKeys.up:
                 if index > 0:
                     index -= 1
             elif keypress in DefaultKeys.down:
@@ -324,9 +348,7 @@ def select_multiple(
                     error_message = f'Must select at least {minimal_count} options'
                 else:
                     break
-            elif keypress in DefaultKeys.interrupt:
-                if Config.raise_on_interrupt:
-                    raise KeyboardInterrupt
+            elif keypress in DefaultKeys.escape:
                 return []
         if return_indices:
             return ticked_indices
@@ -382,27 +404,29 @@ def confirm(
             no_prefix = selected_prefix if no else deselected_prefix
             rendered = f'{question_line}\n{yes_prefix}{yes_text}\n{no_prefix}{no_text}\n\n(Confirm with [bold]enter[/bold])'
             _update_rendered(live, rendered)
-            keypress = readchar.readkey()
 
-            if keypress in DefaultKeys.down or keypress in DefaultKeys.up:
+            keypress = get_key()
+            if keypress in DefaultKeys.interrupt:
+                if Config.raise_on_interrupt:
+                    raise KeyboardInterrupt()
+                return None
+            elif keypress in DefaultKeys.down or keypress in DefaultKeys.up:
                 is_yes = not is_yes
                 is_selected = True
                 current_message = yes_text if is_yes else no_text
-            elif keypress in DefaultKeys.delete:
+            elif keypress in DefaultKeys.backspace:
                 if current_message:
                     current_message = current_message[:-1]
-            elif keypress in DefaultKeys.interrupt:
-                if Config.raise_on_interrupt:
-                    raise KeyboardInterrupt
+            elif keypress in DefaultKeys.escape:
                 return None
             elif keypress in DefaultKeys.confirm:
                 if is_selected:
                     break
-            elif keypress in '\t':
+            elif keypress in DefaultKeys.tab:
                 if is_selected:
                     current_message = yes_text if is_yes else no_text
             else:
-                current_message += keypress
+                current_message += str(keypress)
                 match_yes = yes_text
                 match_no = no_text
                 match_text = current_message
